@@ -16,22 +16,21 @@
 
 import contextlib
 import itertools
+from unittest import mock
 
-import mock
-from oslotest import base as oslo_test_base
-import six
 import sqlalchemy as sqla
 from sqlalchemy.engine import url as sqla_url
 from sqlalchemy import event
 import sqlalchemy.exc
-from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.orm import mapper
+from sqlalchemy.orm import declarative_base
+from sqlalchemy.orm import registry
+from sqlalchemy import sql
 
 from oslo_db import exception
-from oslo_db.sqlalchemy.compat import utils as compat_utils
 from oslo_db.sqlalchemy import engines
 from oslo_db.sqlalchemy import exc_filters
-from oslo_db.tests.sqlalchemy import base as test_base
+from oslo_db.tests import base as test_base
+from oslo_db.tests.sqlalchemy import base as db_test_base
 from oslo_db.tests import utils as test_utils
 
 _TABLE_NAME = '__tmp__test__tmp__'
@@ -54,11 +53,8 @@ class _SQLAExceptionMatcher(object):
             self.assertEqual(exception_type, exc.__class__.__name__)
         if isinstance(message, tuple):
             self.assertEqual(
-                [m.lower()
-                 if isinstance(m, six.string_types) else m for m in message],
-                [a.lower()
-                 if isinstance(a, six.string_types) else a
-                 for a in exc.orig.args]
+                [m.lower() if isinstance(m, str) else m for m in message],
+                [a.lower() if isinstance(a, str) else a for a in exc.orig.args]
             )
         else:
             self.assertEqual(message.lower(), str(exc.orig).lower())
@@ -73,7 +69,7 @@ class _SQLAExceptionMatcher(object):
                 self.assertEqual(sql, exc.statement)
 
 
-class TestsExceptionFilter(_SQLAExceptionMatcher, oslo_test_base.BaseTestCase):
+class TestsExceptionFilter(_SQLAExceptionMatcher, test_base.BaseTestCase):
 
     class Error(Exception):
         """DBAPI base error.
@@ -162,7 +158,7 @@ class TestsExceptionFilter(_SQLAExceptionMatcher, oslo_test_base.BaseTestCase):
         with self._fixture(dialect_name, raises, is_disconnect=is_disconnect):
             with self.engine.connect() as conn:
                 matched = self.assertRaises(
-                    expected, conn.execute, statement, params
+                    expected, conn.execute, sql.text(statement), params
                 )
                 return matched
 
@@ -228,14 +224,14 @@ class TestFallthroughsAndNonDBAPI(TestsExceptionFilter):
         # or at least not documented anywhere.
         uee_ref = None
         try:
-            six.u('\u2435').encode('ascii')
+            '\u2435'.encode('ascii')
         except UnicodeEncodeError as uee:
             # Python3.x added new scoping rules here (sadly)
             # http://legacy.python.org/dev/peps/pep-3110/#semantic-changes
             uee_ref = uee
 
         self._run_test(
-            "postgresql", six.u('select \u2435'),
+            'postgresql', 'select \u2435',
             uee_ref,
             exception.DBInvalidUnicodeParameter
             )
@@ -250,13 +246,14 @@ class TestFallthroughsAndNonDBAPI(TestsExceptionFilter):
 
 
 class TestNonExistentConstraint(
-        _SQLAExceptionMatcher,
-        test_base.DbTestCase):
+    _SQLAExceptionMatcher,
+    db_test_base._DbTestCase,
+):
 
     def setUp(self):
         super(TestNonExistentConstraint, self).setUp()
 
-        meta = sqla.MetaData(bind=self.engine)
+        meta = sqla.MetaData()
 
         self.table_1 = sqla.Table(
             "resource_foo", meta,
@@ -264,22 +261,25 @@ class TestNonExistentConstraint(
             mysql_engine='InnoDB',
             mysql_charset='utf8',
         )
-        self.table_1.create()
+        self.table_1.create(self.engine)
 
 
 class TestNonExistentConstraintPostgreSQL(
-        TestNonExistentConstraint,
-        test_base.PostgreSQLOpportunisticTestCase):
+    TestNonExistentConstraint,
+    db_test_base._PostgreSQLOpportunisticTestCase,
+):
 
     def test_raise(self):
-        matched = self.assertRaises(
-            exception.DBNonExistentConstraint,
-            self.engine.execute,
-            sqla.schema.DropConstraint(
-                sqla.ForeignKeyConstraint(["id"], ["baz.id"],
-                                          name="bar_fkey",
-                                          table=self.table_1)),
-        )
+        with self.engine.connect() as conn:
+            matched = self.assertRaises(
+                exception.DBNonExistentConstraint,
+                conn.execute,
+                sqla.schema.DropConstraint(
+                    sqla.ForeignKeyConstraint(["id"], ["baz.id"],
+                                              name="bar_fkey",
+                                              table=self.table_1)),
+            )
+
         self.assertInnerException(
             matched,
             "ProgrammingError",
@@ -292,18 +292,21 @@ class TestNonExistentConstraintPostgreSQL(
 
 
 class TestNonExistentConstraintMySQL(
-        TestNonExistentConstraint,
-        test_base.MySQLOpportunisticTestCase):
+    TestNonExistentConstraint,
+    db_test_base._MySQLOpportunisticTestCase,
+):
 
     def test_raise(self):
-        matched = self.assertRaises(
-            exception.DBNonExistentConstraint,
-            self.engine.execute,
-            sqla.schema.DropConstraint(
-                sqla.ForeignKeyConstraint(["id"], ["baz.id"],
-                                          name="bar_fkey",
-                                          table=self.table_1)),
-        )
+        with self.engine.connect() as conn:
+            matched = self.assertRaises(
+                exception.DBNonExistentConstraint,
+                conn.execute,
+                sqla.schema.DropConstraint(
+                    sqla.ForeignKeyConstraint(["id"], ["baz.id"],
+                                              name="bar_fkey",
+                                              table=self.table_1)),
+            )
+
         # NOTE(jd) Cannot check precisely with assertInnerException since MySQL
         # error are not the same depending on its version…
         self.assertIsInstance(matched.inner_exception,
@@ -316,13 +319,14 @@ class TestNonExistentConstraintMySQL(
 
 
 class TestNonExistentTable(
-        _SQLAExceptionMatcher,
-        test_base.DbTestCase):
+    _SQLAExceptionMatcher,
+    db_test_base._DbTestCase,
+):
 
     def setUp(self):
         super(TestNonExistentTable, self).setUp()
 
-        self.meta = sqla.MetaData(bind=self.engine)
+        self.meta = sqla.MetaData()
 
         self.table_1 = sqla.Table(
             "foo", self.meta,
@@ -332,11 +336,13 @@ class TestNonExistentTable(
         )
 
     def test_raise(self):
-        matched = self.assertRaises(
-            exception.DBNonExistentTable,
-            self.engine.execute,
-            sqla.schema.DropTable(self.table_1),
-        )
+        with self.engine.connect() as conn:
+            matched = self.assertRaises(
+                exception.DBNonExistentTable,
+                conn.execute,
+                sqla.schema.DropTable(self.table_1),
+            )
+
         self.assertInnerException(
             matched,
             "OperationalError",
@@ -347,15 +353,18 @@ class TestNonExistentTable(
 
 
 class TestNonExistentTablePostgreSQL(
-        TestNonExistentTable,
-        test_base.PostgreSQLOpportunisticTestCase):
+    TestNonExistentTable,
+    db_test_base._PostgreSQLOpportunisticTestCase,
+):
 
     def test_raise(self):
-        matched = self.assertRaises(
-            exception.DBNonExistentTable,
-            self.engine.execute,
-            sqla.schema.DropTable(self.table_1),
-        )
+        with self.engine.connect() as conn:
+            matched = self.assertRaises(
+                exception.DBNonExistentTable,
+                conn.execute,
+                sqla.schema.DropTable(self.table_1),
+            )
+
         self.assertInnerException(
             matched,
             "ProgrammingError",
@@ -366,15 +375,18 @@ class TestNonExistentTablePostgreSQL(
 
 
 class TestNonExistentTableMySQL(
-        TestNonExistentTable,
-        test_base.MySQLOpportunisticTestCase):
+    TestNonExistentTable,
+    db_test_base._MySQLOpportunisticTestCase,
+):
 
     def test_raise(self):
-        matched = self.assertRaises(
-            exception.DBNonExistentTable,
-            self.engine.execute,
-            sqla.schema.DropTable(self.table_1),
-        )
+        with self.engine.connect() as conn:
+            matched = self.assertRaises(
+                exception.DBNonExistentTable,
+                conn.execute,
+                sqla.schema.DropTable(self.table_1),
+            )
+
         # NOTE(jd) Cannot check precisely with assertInnerException since MySQL
         # error are not the same depending on its version…
         self.assertIsInstance(matched.inner_exception,
@@ -384,15 +396,24 @@ class TestNonExistentTableMySQL(
 
 
 class TestNonExistentDatabase(
-        _SQLAExceptionMatcher,
-        test_base.DbTestCase):
+    _SQLAExceptionMatcher,
+    db_test_base._DbTestCase,
+):
 
     def setUp(self):
         super(TestNonExistentDatabase, self).setUp()
 
         url = sqla_url.make_url(str(self.engine.url))
-        url.database = 'non_existent_database'
-        self.url = url
+
+        # TODO(zzzeek): remove hasattr() conditional in favor of "url.set()"
+        # when SQLAlchemy 1.4 is the minimum version in requirements
+        if hasattr(url, "set"):
+            self.url = url.set(database="non_existent_database")
+        else:
+            # TODO(zzzeek): remove when SQLAlchemy 1.4
+            # is the minimum version in requirements
+            url.database = 'non_existent_database'
+            self.url = url
 
     def test_raise(self):
         matched = self.assertRaises(
@@ -410,8 +431,9 @@ class TestNonExistentDatabase(
 
 
 class TestNonExistentDatabaseMySQL(
-        TestNonExistentDatabase,
-        test_base.MySQLOpportunisticTestCase):
+    TestNonExistentDatabase,
+    db_test_base._MySQLOpportunisticTestCase,
+):
 
     def test_raise(self):
         matched = self.assertRaises(
@@ -429,8 +451,9 @@ class TestNonExistentDatabaseMySQL(
 
 
 class TestNonExistentDatabasePostgreSQL(
-        TestNonExistentDatabase,
-        test_base.PostgreSQLOpportunisticTestCase):
+    TestNonExistentDatabase,
+    db_test_base._PostgreSQLOpportunisticTestCase,
+):
 
     def test_raise(self):
         matched = self.assertRaises(
@@ -446,12 +469,14 @@ class TestNonExistentDatabasePostgreSQL(
         )
 
 
-class TestReferenceErrorSQLite(_SQLAExceptionMatcher, test_base.DbTestCase):
+class TestReferenceErrorSQLite(
+    _SQLAExceptionMatcher, db_test_base._DbTestCase,
+):
 
     def setUp(self):
         super(TestReferenceErrorSQLite, self).setUp()
 
-        meta = sqla.MetaData(bind=self.engine)
+        meta = sqla.MetaData()
 
         self.table_1 = sqla.Table(
             "resource_foo", meta,
@@ -460,7 +485,7 @@ class TestReferenceErrorSQLite(_SQLAExceptionMatcher, test_base.DbTestCase):
             mysql_engine='InnoDB',
             mysql_charset='utf8',
         )
-        self.table_1.create()
+        self.table_1.create(self.engine)
 
         self.table_2 = sqla.Table(
             "resource_entity", meta,
@@ -470,16 +495,23 @@ class TestReferenceErrorSQLite(_SQLAExceptionMatcher, test_base.DbTestCase):
             mysql_engine='InnoDB',
             mysql_charset='utf8',
         )
-        self.table_2.create()
+        self.table_2.create(self.engine)
 
     def test_raise(self):
-        self.engine.execute("PRAGMA foreign_keys = ON;")
+        connection = self.engine.raw_connection()
+        try:
+            cursor = connection.cursor()
+            cursor.execute('PRAGMA foreign_keys = ON')
+            cursor.close()
+        finally:
+            connection.close()
 
-        matched = self.assertRaises(
-            exception.DBReferenceError,
-            self.engine.execute,
-            self.table_2.insert({'id': 1, 'foo_id': 2})
-        )
+        with self.engine.connect() as conn:
+            matched = self.assertRaises(
+                exception.DBReferenceError,
+                conn.execute,
+                self.table_2.insert().values(id=1, foo_id=2)
+            )
 
         self.assertInnerException(
             matched,
@@ -495,16 +527,25 @@ class TestReferenceErrorSQLite(_SQLAExceptionMatcher, test_base.DbTestCase):
         self.assertIsNone(matched.key_table)
 
     def test_raise_delete(self):
-        self.engine.execute("PRAGMA foreign_keys = ON;")
+        connection = self.engine.raw_connection()
+        try:
+            cursor = connection.cursor()
+            cursor.execute('PRAGMA foreign_keys = ON')
+            cursor.close()
+        finally:
+            connection.close()
 
         with self.engine.connect() as conn:
-            conn.execute(self.table_1.insert({"id": 1234, "foo": 42}))
-            conn.execute(self.table_2.insert({"id": 4321, "foo_id": 1234}))
-        matched = self.assertRaises(
-            exception.DBReferenceError,
-            self.engine.execute,
-            self.table_1.delete()
-        )
+            with conn.begin():
+                conn.execute(self.table_1.insert().values(id=1234, foo=42))
+                conn.execute(
+                    self.table_2.insert().values(id=4321, foo_id=1234))
+                matched = self.assertRaises(
+                    exception.DBReferenceError,
+                    conn.execute,
+                    self.table_1.delete()
+                )
+
         self.assertInnerException(
             matched,
             "IntegrityError",
@@ -519,15 +560,19 @@ class TestReferenceErrorSQLite(_SQLAExceptionMatcher, test_base.DbTestCase):
         self.assertIsNone(matched.key_table)
 
 
-class TestReferenceErrorPostgreSQL(TestReferenceErrorSQLite,
-                                   test_base.PostgreSQLOpportunisticTestCase):
+class TestReferenceErrorPostgreSQL(
+    TestReferenceErrorSQLite,
+    db_test_base._PostgreSQLOpportunisticTestCase,
+):
     def test_raise(self):
-        params = {'id': 1, 'foo_id': 2}
-        matched = self.assertRaises(
-            exception.DBReferenceError,
-            self.engine.execute,
-            self.table_2.insert(params)
-        )
+        with self.engine.connect() as conn:
+            params = {'id': 1, 'foo_id': 2}
+            matched = self.assertRaises(
+                exception.DBReferenceError,
+                conn.execute,
+                self.table_2.insert().values(**params)
+            )
+
         self.assertInnerException(
             matched,
             "IntegrityError",
@@ -546,13 +591,18 @@ class TestReferenceErrorPostgreSQL(TestReferenceErrorSQLite,
 
     def test_raise_delete(self):
         with self.engine.connect() as conn:
-            conn.execute(self.table_1.insert({"id": 1234, "foo": 42}))
-            conn.execute(self.table_2.insert({"id": 4321, "foo_id": 1234}))
-        matched = self.assertRaises(
-            exception.DBReferenceError,
-            self.engine.execute,
-            self.table_1.delete()
-        )
+            with conn.begin():
+                conn.execute(self.table_1.insert().values(id=1234, foo=42))
+                conn.execute(
+                    self.table_2.insert().values(id=4321, foo_id=1234))
+
+            with conn.begin():
+                matched = self.assertRaises(
+                    exception.DBReferenceError,
+                    conn.execute,
+                    self.table_1.delete()
+                )
+
         self.assertInnerException(
             matched,
             "IntegrityError",
@@ -570,14 +620,17 @@ class TestReferenceErrorPostgreSQL(TestReferenceErrorSQLite,
         self.assertEqual("resource_entity", matched.key_table)
 
 
-class TestReferenceErrorMySQL(TestReferenceErrorSQLite,
-                              test_base.MySQLOpportunisticTestCase):
+class TestReferenceErrorMySQL(
+    TestReferenceErrorSQLite,
+    db_test_base._MySQLOpportunisticTestCase,
+):
     def test_raise(self):
-        matched = self.assertRaises(
-            exception.DBReferenceError,
-            self.engine.execute,
-            self.table_2.insert({'id': 1, 'foo_id': 2})
-        )
+        with self.engine.connect() as conn:
+            matched = self.assertRaises(
+                exception.DBReferenceError,
+                conn.execute,
+                self.table_2.insert().values(id=1, foo_id=2)
+            )
 
         # NOTE(jd) Cannot check precisely with assertInnerException since MySQL
         # error are not the same depending on its version…
@@ -594,12 +647,12 @@ class TestReferenceErrorMySQL(TestReferenceErrorSQLite,
             conn.detach()  # will not be returned to the pool when closed
 
             # this is incompatible with some internals of the engine
-            conn.execute("SET SESSION sql_mode = 'ANSI';")
+            conn.execute(sql.text("SET SESSION sql_mode = 'ANSI';"))
 
             matched = self.assertRaises(
                 exception.DBReferenceError,
                 conn.execute,
-                self.table_2.insert({'id': 1, 'foo_id': 2})
+                self.table_2.insert().values(id=1, foo_id=2)
             )
 
         # NOTE(jd) Cannot check precisely with assertInnerException since MySQL
@@ -613,14 +666,14 @@ class TestReferenceErrorMySQL(TestReferenceErrorSQLite,
         self.assertEqual("resource_foo", matched.key_table)
 
     def test_raise_delete(self):
-        with self.engine.connect() as conn:
-            conn.execute(self.table_1.insert({"id": 1234, "foo": 42}))
-            conn.execute(self.table_2.insert({"id": 4321, "foo_id": 1234}))
-        matched = self.assertRaises(
-            exception.DBReferenceError,
-            self.engine.execute,
-            self.table_1.delete()
-        )
+        with self.engine.connect() as conn, conn.begin():
+            conn.execute(self.table_1.insert().values(id=1234, foo=42))
+            conn.execute(self.table_2.insert().values(id=4321, foo_id=1234))
+            matched = self.assertRaises(
+                exception.DBReferenceError,
+                conn.execute,
+                self.table_1.delete()
+            )
         # NOTE(jd) Cannot check precisely with assertInnerException since MySQL
         # error are not the same depending on its version…
         self.assertIsInstance(matched.inner_exception,
@@ -632,7 +685,9 @@ class TestReferenceErrorMySQL(TestReferenceErrorSQLite,
         self.assertEqual("resource_foo", matched.key_table)
 
 
-class TestExceptionCauseMySQLSavepoint(test_base.MySQLOpportunisticTestCase):
+class TestExceptionCauseMySQLSavepoint(
+    db_test_base._MySQLOpportunisticTestCase,
+):
     def setUp(self):
         super(TestExceptionCauseMySQLSavepoint, self).setUp()
 
@@ -655,55 +710,24 @@ class TestExceptionCauseMySQLSavepoint(test_base.MySQLOpportunisticTestCase):
         with session.begin():
             session.add(self.A(id=1))
         try:
-
             with session.begin():
-
                 try:
                     with session.begin_nested():
-                        session.execute("rollback")
+                        session.execute(sql.text("rollback"))
                         session.add(self.A(id=1))
-
                 # outermost is the failed SAVEPOINT rollback
                 # from the "with session.begin_nested()"
                 except exception.DBError as dbe_inner:
-
-                    if not compat_utils.sqla_110:
-                        # first "cause" is the failed SAVEPOINT rollback
-                        # from inside of flush(), when it fails
-                        self.assertTrue(
-                            isinstance(
-                                dbe_inner.cause,
-                                exception.DBError
-                            )
-                        )
-
-                        # second "cause" is then the actual DB duplicate
-                        self.assertTrue(
-                            isinstance(
-                                dbe_inner.cause.cause,
-                                exception.DBDuplicateEntry
-                            )
-                        )
-                    else:
-                        # in SQLA 1.1, the rollback() method of Session
-                        # catches the error and repairs the state of the
-                        # session even though the SAVEPOINT was lost;
-                        # the net result here is that one exception is thrown
-                        # instead of two.  This is SQLAlchemy ticket #3680
-                        self.assertTrue(
-                            isinstance(
-                                dbe_inner.cause,
-                                exception.DBDuplicateEntry
-                            )
-                        )
+                    # in SQLA 1.1+, the rollback() method of Session
+                    # catches the error and repairs the state of the
+                    # session even though the SAVEPOINT was lost;
+                    # the net result here is that one exception is thrown
+                    # instead of two.  This is SQLAlchemy ticket #3680
+                    self.assertIsInstance(
+                        dbe_inner.cause, exception.DBDuplicateEntry)
 
         except exception.DBError as dbe_outer:
-            self.assertTrue(
-                isinstance(
-                    dbe_outer.cause,
-                    exception.DBDuplicateEntry
-                )
-            )
+            self.AssertIsInstance(dbe_outer.cause, exception.DBDuplicateEntry)
 
         # resets itself afterwards
         try:
@@ -717,7 +741,7 @@ class TestExceptionCauseMySQLSavepoint(test_base.MySQLOpportunisticTestCase):
 
         session.begin()
         try:
-            session.execute("select 1")
+            session.execute(sql.text("select 1"))
 
             # close underying DB connection
             session.connection().connection.connection.close()
@@ -727,7 +751,7 @@ class TestExceptionCauseMySQLSavepoint(test_base.MySQLOpportunisticTestCase):
             # session.execute("kill connection %s" % conn_id)
 
             # try using it, will raise an error
-            session.execute("select 1")
+            session.execute(sql.text("select 1"))
         except exception.DBConnectionError:
             # issue being tested is that this session.rollback()
             # does not itself try to re-connect and raise another
@@ -742,7 +766,7 @@ class TestExceptionCauseMySQLSavepoint(test_base.MySQLOpportunisticTestCase):
         session.begin()
         try:
             session.begin_nested()
-            session.execute("select 1")
+            session.execute(sql.text("select 1"))
 
             # close underying DB connection
             session.connection().connection.connection.close()
@@ -752,7 +776,7 @@ class TestExceptionCauseMySQLSavepoint(test_base.MySQLOpportunisticTestCase):
             # session.execute("kill connection %s" % conn_id)
 
             # try using it, will raise an error
-            session.execute("select 1")
+            session.execute(sql.text("select 1"))
         except exception.DBConnectionError:
             # issue being tested is that this session.rollback()
             # does not itself try to re-connect and raise another
@@ -762,42 +786,6 @@ class TestExceptionCauseMySQLSavepoint(test_base.MySQLOpportunisticTestCase):
             assert False, "no exception raised"
 
 
-class TestDBDataErrorSQLite(_SQLAExceptionMatcher, test_base.DbTestCase):
-
-    def setUp(self):
-        super(TestDBDataErrorSQLite, self).setUp()
-
-        if six.PY3:
-            self.skipTest("SQLite database supports unicode value for python3")
-
-        meta = sqla.MetaData(bind=self.engine)
-
-        self.table_1 = sqla.Table(
-            "resource_foo", meta,
-            sqla.Column("name", sqla.String),
-        )
-        self.table_1.create()
-
-    def test_raise(self):
-
-        matched = self.assertRaises(
-            exception.DBDataError,
-            self.engine.execute,
-            self.table_1.insert({'name': u'\u2713'.encode('utf-8')})
-        )
-
-        self.assertInnerException(
-            matched,
-            "ProgrammingError",
-            "You must not use 8-bit bytestrings unless you use a "
-            "text_factory that can interpret 8-bit bytestrings "
-            "(like text_factory = str). It is highly recommended that "
-            "you instead just switch your application to Unicode strings.",
-            "INSERT INTO resource_foo (name) VALUES (?)",
-            (u'\u2713'.encode('utf-8'),)
-        )
-
-
 class TestConstraint(TestsExceptionFilter):
     def test_postgresql(self):
         matched = self._run_test(
@@ -885,6 +873,14 @@ class TestDuplicate(TestsExceptionFilter):
             expected_value='2'
         )
 
+    def test_mysql_duplicate_entry_key_start_with_tablename(self):
+        self._run_dupe_constraint_test(
+            "mysql",
+            "1062 (23000): Duplicate entry '2' for key 'tbl.uniq_tbl0b'",
+            expected_columns=['b'],
+            expected_value='2'
+        )
+
     def test_mysql_binary(self):
         self._run_dupe_constraint_test(
             "mysql",
@@ -903,6 +899,24 @@ class TestDuplicate(TestsExceptionFilter):
             expected_value="'\\\\x8A$\\\\x8D\\\\xA6\"s\\\\x8E!,"
         )
 
+    def test_mysql_duplicate_entry_key_start_with_tablename_binary(self):
+        self._run_dupe_constraint_test(
+            "mysql",
+            "(1062, \'Duplicate entry "
+            "\\\'\\\\x8A$\\\\x8D\\\\xA6\"s\\\\x8E\\\' "
+            "for key \\\'tbl.uniq_tbl0c1\\\'\')",
+            expected_columns=['c1'],
+            expected_value="\\\\x8A$\\\\x8D\\\\xA6\"s\\\\x8E"
+        )
+        self._run_dupe_constraint_test(
+            "mysql",
+            "(1062, \'Duplicate entry "
+            "''\\\\x8A$\\\\x8D\\\\xA6\"s\\\\x8E!,' "
+            "for key 'tbl.uniq_tbl0c1'\')",
+            expected_columns=['c1'],
+            expected_value="'\\\\x8A$\\\\x8D\\\\xA6\"s\\\\x8E!,"
+        )
+
     def test_postgresql_single(self):
         self._run_dupe_constraint_test(
             'postgresql',
@@ -919,28 +933,6 @@ class TestDuplicate(TestsExceptionFilter):
             exception.DBError
         )
 
-    def test_ibm_db_sa(self):
-        self._run_dupe_constraint_test(
-            'ibm_db_sa',
-            'SQL0803N  One or more values in the INSERT statement, UPDATE '
-            'statement, or foreign key update caused by a DELETE statement are'
-            ' not valid because the primary key, unique constraint or unique '
-            'index identified by "2" constrains table "NOVA.KEY_PAIRS" from '
-            'having duplicate values for the index key.',
-            expected_columns=[]
-        )
-
-    def test_ibm_db_sa_notadupe(self):
-        self._not_dupe_constraint_test(
-            'ibm_db_sa',
-            'ALTER TABLE instance_types ADD CONSTRAINT '
-            'uniq_name_x_deleted UNIQUE (name, deleted)',
-            'SQL0542N  The column named "NAME" cannot be a column of a '
-            'primary key or unique key constraint because it can contain null '
-            'values.',
-            exception.DBError
-        )
-
 
 class TestDeadlock(TestsExceptionFilter):
     statement = ('SELECT quota_usages.created_at AS '
@@ -995,6 +987,12 @@ class TestDeadlock(TestsExceptionFilter):
             orig_exception_cls=self.InternalError
         )
 
+        self._run_deadlock_detect_test(
+            "mysql",
+            "(1213, 'Deadlock: wsrep aborted transaction')",
+            orig_exception_cls=self.InternalError
+        )
+
     def test_mysql_pymysql_galera_deadlock(self):
         self._run_deadlock_detect_test(
             "mysql",
@@ -1036,25 +1034,6 @@ class TestDeadlock(TestsExceptionFilter):
             orig_exception_cls=self.TransactionRollbackError
         )
 
-    def test_ibm_db_sa_deadlock(self):
-        self._run_deadlock_detect_test(
-            "ibm_db_sa",
-            "SQL0911N The current transaction has been "
-            "rolled back because of a deadlock or timeout",
-            # use the lowest class b.c. I don't know what actual error
-            # class DB2's driver would raise for this
-            orig_exception_cls=self.Error
-        )
-
-    def test_ibm_db_sa_not_deadlock(self):
-        self._not_deadlock_test(
-            "ibm_db_sa",
-            "SQL01234B Some other error.",
-            exception.DBError,
-            "Error",
-            orig_exception_cls=self.Error
-        )
-
 
 class TestDataError(TestsExceptionFilter):
     def _run_bad_data_test(self, dialect_name, message, error_class):
@@ -1084,7 +1063,7 @@ class TestDataError(TestsExceptionFilter):
                                 self.DataError)
 
 
-class IntegrationTest(test_base.DbTestCase):
+class IntegrationTest(db_test_base._DbTestCase):
     """Test an actual error-raising round trips against the database."""
 
     def setUp(self):
@@ -1101,10 +1080,13 @@ class IntegrationTest(test_base.DbTestCase):
         self.test_table.create(self.engine)
         self.addCleanup(self.test_table.drop, self.engine)
 
+        reg = registry()
+
         class Foo(object):
             def __init__(self, counter):
                 self.counter = counter
-        mapper(Foo, self.test_table)
+
+        reg.map_imperatively(Foo, self.test_table)
         self.Foo = Foo
 
     def test_flush_wrapper_duplicate_entry(self):
@@ -1217,11 +1199,14 @@ class TestDBDisconnected(TestsExceptionFilter):
                 yield
 
     def _test_ping_listener_disconnected(
-            self, dialect_name, exc_obj, is_disconnect=True):
+        self, dialect_name, exc_obj, is_disconnect=True,
+    ):
         with self._fixture(dialect_name, exc_obj, 1, is_disconnect):
             conn = self.engine.connect()
             with conn.begin():
-                self.assertEqual(1, conn.scalar(sqla.select([1])))
+                self.assertEqual(
+                    1, conn.execute(sqla.select(1)).scalars().first(),
+                )
                 self.assertFalse(conn.closed)
                 self.assertFalse(conn.invalidated)
                 self.assertTrue(conn.in_transaction())
@@ -1234,7 +1219,10 @@ class TestDBDisconnected(TestsExceptionFilter):
 
         # test implicit execution
         with self._fixture(dialect_name, exc_obj, 1):
-            self.assertEqual(1, self.engine.scalar(sqla.select([1])))
+            with self.engine.connect() as conn:
+                self.assertEqual(
+                    1, conn.execute(sqla.select(1)).scalars().first(),
+                )
 
     def test_mariadb_error_1927(self):
         for code in [1927]:
@@ -1288,21 +1276,6 @@ class TestDBDisconnected(TestsExceptionFilter):
             is_disconnect=False
         )
 
-    def test_db2_ping_listener_disconnected(self):
-        self._test_ping_listener_disconnected(
-            "ibm_db_sa",
-            self.OperationalError(
-                'SQL30081N: DB2 Server connection is no longer active')
-        )
-
-    def test_db2_ping_listener_disconnected_regex_only(self):
-        self._test_ping_listener_disconnected(
-            "ibm_db_sa",
-            self.OperationalError(
-                'SQL30081N: DB2 Server connection is no longer active'),
-            is_disconnect=False
-        )
-
     def test_postgresql_ping_listener_disconnected(self):
         self._test_ping_listener_disconnected(
             "postgresql",
@@ -1357,7 +1330,7 @@ class TestDBConnectRetry(TestsExceptionFilter):
             2, -1
         )
         # conn is good
-        self.assertEqual(1, conn.scalar(sqla.select([1])))
+        self.assertEqual(1, conn.scalar(sqla.select(1)))
 
     def test_connect_retry_past_failure(self):
         conn = self._run_test(
@@ -1366,7 +1339,7 @@ class TestDBConnectRetry(TestsExceptionFilter):
             2, 3
         )
         # conn is good
-        self.assertEqual(1, conn.scalar(sqla.select([1])))
+        self.assertEqual(1, conn.scalar(sqla.select(1)))
 
     def test_connect_retry_not_candidate_exception(self):
         self.assertRaises(
@@ -1387,24 +1360,6 @@ class TestDBConnectRetry(TestsExceptionFilter):
             3, 2
         )
 
-    def test_db2_error_positive(self):
-        conn = self._run_test(
-            "ibm_db_sa",
-            self.OperationalError("blah blah -30081 blah blah"),
-            2, -1
-        )
-        # conn is good
-        self.assertEqual(1, conn.scalar(sqla.select([1])))
-
-    def test_db2_error_negative(self):
-        self.assertRaises(
-            sqla.exc.OperationalError,
-            self._run_test,
-            "ibm_db_sa",
-            self.OperationalError("blah blah -39981 blah blah"),
-            2, 3
-        )
-
 
 class TestDBConnectPingWrapping(TestsExceptionFilter):
 
@@ -1451,7 +1406,7 @@ class TestDBConnectPingWrapping(TestsExceptionFilter):
             self, dialect_name, exc_obj, is_disconnect=True):
         with self._fixture(dialect_name, exc_obj, 3, is_disconnect):
             conn = self.engine.connect()
-            self.assertEqual(1, conn.scalar(sqla.select([1])))
+            self.assertEqual(1, conn.scalar(sqla.select(1)))
             conn.close()
 
         with self._fixture(dialect_name, exc_obj, 1, is_disconnect):
@@ -1471,15 +1426,15 @@ class TestDBConnectPingWrapping(TestsExceptionFilter):
         with self._fixture(dialect_name, exc_obj, 1, is_disconnect):
             self.assertRaises(
                 exception.DBConnectionError,
-                self.engine.contextual_connect
+                self.engine.connect
             )
             self.assertRaises(
                 exception.DBConnectionError,
-                self.engine.contextual_connect
+                self.engine.connect
             )
             self.assertRaises(
                 exception.DBConnectionError,
-                self.engine.contextual_connect
+                self.engine.connect
             )
 
     def test_mysql_w_disconnect_flag(self):
